/****************************************************************************
    
    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef NEWTONATOR_H_INCLUDED
#define NEWTONATOR_H_INCLUDED

#include "VariSource.h"

class Newtonator
{
    public:
    Newtonator(bool useNoteEnvelope, unsigned int channels = 2);
    virtual ~Newtonator();
    inline VariSource* getFloorSources() { return _floorSource; };
    inline VariSource* getGravModSources() { return _gravModSource; };
    inline stk::ADSR* getAmpEnvs() { return _ampEnv; };
    stk::StkFloat newtTick(unsigned int channel, unsigned int frameIdx);
    virtual void initChannels(unsigned int numChannels);

    //Note control
    bool isPlaying();
    void keyOn(stk::StkFloat velocity);
    void keyOff(stk::StkFloat velocity);

    //Accessors for user control variables
    inline void setFloorAttraction(stk::StkFloat val) { _floorAttraction = pow(val, 4) / 4.0; };
    inline stk::StkFloat getFloorAttraction() { return _floorAttraction; };
    inline void setFloorScale(stk::StkFloat val) { _floorScale = val; };
    inline stk::StkFloat getFloorScale() { return _floorScale; };
    inline void setGravModDepth(stk::StkFloat val) { _gravModDepth = val; };
    inline stk::StkFloat getGravModDepth() { return _gravModDepth; };
    inline void setGravity(stk::StkFloat val) { _gravity = val * 0.003051758; };
    inline stk::StkFloat getGravity() { return _gravity; };
    inline void setGravityScale(stk::StkFloat val) { _gravScale = (1.0000001 - sqrt(val)) * 2.0; };
    inline stk::StkFloat getGravityScale() { return _gravScale; };
    inline void setClip(bool val) { _clip = val; };
    inline bool getClip() { return _clip; };

    private:
    stk::StkFloat* _pPrevPos;
    stk::StkFloat* _pPrevFloor;
    stk::StkFloat* _pPrevVeloc;
    int* _gravDir;
    unsigned int _numChannels;
    const stk::StkFloat MAX_VAL;
    const stk::StkFloat MIN_VAL;
    bool _usingNoteEnvelope;

    //User Control values
    stk::StkFloat _floorAttraction;
    stk::StkFloat _floorScale;
    stk::StkFloat _gravModDepth;
    stk::StkFloat _gravity;
    stk::StkFloat _gravScale; //can't ever reach zero
    bool _clip;
    VariSource* _floorSource;
    VariSource* _gravModSource;
    stk::ADSR* _ampEnv;
    stk::StkFloat _noteAmp;

    //Methods
    void clearChannelMemory();
};

#endif // NEWTONATOR_H_INCLUDED
